@@ -15,14 +15,12 @@
 
 //! Integration tests that use golden input and output files.
 
+// TODO(jmmv): All of the supporting code here is duplicated in the other EndBASIC crates.
+// Should probably add a separate testutils crate, but I'm hesitant for now.
+
 // Keep these in sync with other top-level files.
 #![warn(anonymous_parameters, bad_style, missing_docs)]
-#![warn(
-    unused,
-    unused_extern_crates,
-    unused_import_braces,
-    unused_qualifications
-)]
+#![warn(unused, unused_extern_crates, unused_import_braces, unused_qualifications)]
 #![warn(unsafe_code)]
 
 use std::env;
@@ -49,18 +47,14 @@ fn self_dir() -> PathBuf {
 fn bin_path<P: AsRef<Path>>(name: P) -> PathBuf {
     let test_dir = self_dir();
     let debug_or_release_dir = test_dir.parent().expect("Failed to get parent directory");
-    debug_or_release_dir
-        .join(name)
-        .with_extension(env::consts::EXE_EXTENSION)
+    debug_or_release_dir.join(name).with_extension(env::consts::EXE_EXTENSION)
 }
 
 /// Computes the path to the source file `name`.
 fn src_path(name: &str) -> PathBuf {
     let test_dir = self_dir();
     let debug_or_release_dir = test_dir.parent().expect("Failed to get parent directory");
-    let target_dir = debug_or_release_dir
-        .parent()
-        .expect("Failed to get parent directory");
+    let target_dir = debug_or_release_dir.parent().expect("Failed to get parent directory");
     let dir = target_dir.parent().expect("Failed to get parent directory");
 
     // Sanity-check that we landed in the right location.
@@ -71,10 +65,7 @@ fn src_path(name: &str) -> PathBuf {
 
 /// Same as `src_path` but returns a string reference for the few places where we need this.
 fn src_str(p: &str) -> String {
-    src_path(p)
-        .to_str()
-        .expect("Need paths to be valid strings")
-        .to_owned()
+    src_path(p).to_str().expect("Need paths to be valid strings").to_owned()
 }
 
 /// Describes the behavior for one of the three streams (stdin, stdout, stderr) connected to a
@@ -96,14 +87,9 @@ enum Behavior {
 fn read_golden(path: &Path) -> String {
     let mut f = File::open(path).expect("Failed to open golden data file");
     let mut golden = vec![];
-    f.read_to_end(&mut golden)
-        .expect("Failed to read golden data file");
+    f.read_to_end(&mut golden).expect("Failed to read golden data file");
     let raw = String::from_utf8(golden).expect("Golden data file is not valid UTF-8");
-    let golden = if cfg!(target_os = "windows") {
-        raw.replace("\r\n", "\n")
-    } else {
-        raw
-    };
+    let golden = if cfg!(target_os = "windows") { raw.replace("\r\n", "\n") } else { raw };
 
     // This is the opposite of apply_mocks and ensures we don't leak actual values into the golden
     // files by mistake.
@@ -114,11 +100,7 @@ fn read_golden(path: &Path) -> String {
         path.display()
     );
     let date_re = regex::Regex::new(DATE_RE).unwrap();
-    assert!(
-        !date_re.is_match(&golden),
-        "Golden file {} contains a date",
-        path.display()
-    );
+    assert!(!date_re.is_match(&golden), "Golden file {} contains a date", path.display());
 
     golden
 }
@@ -166,12 +148,11 @@ fn check<P: AsRef<Path>>(
     let result = process::Command::new(bin.as_ref())
         .args(args)
         .stdin(golden_stdin)
+        .env("LINES", "24")
+        .env("COLUMNS", "80")
         .output()
         .expect("Failed to execute subprocess");
-    let code = result
-        .status
-        .code()
-        .expect("Subprocess didn't exit cleanly");
+    let code = result.status.code().expect("Subprocess didn't exit cleanly");
     let stdout =
         apply_mocks(String::from_utf8(result.stdout).expect("Stdout not is not valid UTF-8"));
     let stderr =
@@ -195,7 +176,7 @@ fn test_cli_help() {
             args,
             0,
             Behavior::Null,
-            Behavior::File(src_path("tests/cli/help.out")),
+            Behavior::File(src_path("cli/tests/cli/help.out")),
             Behavior::Null,
         );
     }
@@ -222,9 +203,7 @@ fn test_cli_program_name_uses_arg0() {
     }
 
     let original = bin_path("endbasic");
-    let custom = self_dir()
-        .join("custom-name")
-        .with_extension(env::consts::EXE_EXTENSION);
+    let custom = self_dir().join("custom-name").with_extension(env::consts::EXE_EXTENSION);
     let _delete_custom = DeleteOnDrop { path: &custom };
     fs::copy(&original, &custom).unwrap();
     check(
@@ -278,7 +257,7 @@ fn test_cli_version() {
             args,
             0,
             Behavior::Null,
-            Behavior::File(src_path("tests/cli/version.out")),
+            Behavior::File(src_path("cli/tests/cli/version.out")),
             Behavior::Null,
         );
     }
@@ -287,133 +266,13 @@ fn test_cli_version() {
 }
 
 #[test]
-fn test_example_custom_commands() {
+fn test_repl_console() {
     check(
-        bin_path("examples/custom-commands"),
-        &[],
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/repl/console.bas")],
         0,
         Behavior::Null,
-        Behavior::File(src_path("examples/custom-commands.out")),
-        Behavior::Null,
-    );
-}
-
-#[test]
-fn test_example_minimal() {
-    check(
-        bin_path("examples/minimal"),
-        &[],
-        0,
-        Behavior::Null,
-        Behavior::File(src_path("examples/minimal.out")),
-        Behavior::Null,
-    );
-}
-
-#[test]
-fn test_lang_control_flow() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/control-flow.bas")],
-        0,
-        Behavior::Null,
-        Behavior::File(src_path("tests/lang/control-flow.out")),
-        Behavior::Null,
-    );
-}
-
-#[test]
-fn test_lang_exec_error() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/exec-error.bas")],
-        1,
-        Behavior::Null,
-        Behavior::File(src_path("tests/lang/exec-error.out")),
-        Behavior::File(src_path("tests/lang/exec-error.err")),
-    );
-}
-
-#[test]
-fn test_lang_hello() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/hello.bas")],
-        0,
-        Behavior::Null,
-        Behavior::File(src_path("tests/lang/hello.out")),
-        Behavior::Null,
-    );
-}
-
-#[test]
-fn test_lang_lexer_error() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/lexer-error.bas")],
-        1,
-        Behavior::Null,
-        Behavior::File(src_path("tests/lang/lexer-error.out")),
-        Behavior::File(src_path("tests/lang/lexer-error.err")),
-    );
-}
-
-#[test]
-fn test_lang_no_repl_commands() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/no-repl-commands.bas")],
-        1,
-        Behavior::Null,
-        Behavior::File(src_path("tests/lang/no-repl-commands.out")),
-        Behavior::File(src_path("tests/lang/no-repl-commands.err")),
-    );
-}
-
-#[test]
-fn test_lang_parser_error() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/parser-error.bas")],
-        1,
-        Behavior::Null,
-        Behavior::File(src_path("tests/lang/parser-error.out")),
-        Behavior::File(src_path("tests/lang/parser-error.err")),
-    );
-}
-
-#[test]
-fn test_lang_types() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/types.bas")],
-        0,
-        Behavior::Null,
-        Behavior::File(src_path("tests/lang/types.out")),
-        Behavior::Null,
-    );
-}
-
-#[test]
-fn test_lang_utf8() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/utf8.bas")],
-        0,
-        Behavior::File(src_path("tests/lang/utf8.in")),
-        Behavior::File(src_path("tests/lang/utf8.out")),
-        Behavior::Null,
-    );
-}
-
-#[test]
-fn test_lang_yes_no() {
-    check(
-        bin_path("endbasic"),
-        &[&src_str("tests/lang/yes-no.bas")],
-        0,
-        Behavior::File(src_path("tests/lang/yes-no.in")),
-        Behavior::File(src_path("tests/lang/yes-no.out")),
+        Behavior::File(src_path("cli/tests/repl/console.out")),
         Behavior::Null,
     );
 }
@@ -426,8 +285,8 @@ fn test_repl_dir() {
         bin_path("endbasic"),
         &["--programs-dir", subdir.to_str().unwrap()],
         0,
-        Behavior::File(src_path("tests/repl/dir.in")),
-        Behavior::File(src_path("tests/repl/dir.out")),
+        Behavior::File(src_path("cli/tests/repl/dir.in")),
+        Behavior::File(src_path("cli/tests/repl/dir.out")),
         Behavior::Null,
     );
 }
@@ -438,8 +297,29 @@ fn test_repl_editor() {
         bin_path("endbasic"),
         &[],
         0,
-        Behavior::File(src_path("tests/repl/editor.in")),
-        Behavior::File(src_path("tests/repl/editor.out")),
+        Behavior::File(src_path("cli/tests/repl/editor.in")),
+        Behavior::File(src_path("cli/tests/repl/editor.out")),
+        Behavior::Null,
+    );
+}
+
+#[test]
+fn test_repl_exit() {
+    check(
+        bin_path("endbasic"),
+        &[&src_str("cli/tests/repl/exit.bas")],
+        78,
+        Behavior::Null,
+        Behavior::Null,
+        Behavior::Null,
+    );
+
+    check(
+        bin_path("endbasic"),
+        &[],
+        78,
+        Behavior::File(src_path("cli/tests/repl/exit.bas")),
+        Behavior::File(src_path("cli/tests/repl/exit.out")),
         Behavior::Null,
     );
 }
@@ -450,8 +330,8 @@ fn test_repl_help() {
         bin_path("endbasic"),
         &[],
         0,
-        Behavior::File(src_path("tests/repl/help.in")),
-        Behavior::File(src_path("tests/repl/help.out")),
+        Behavior::File(src_path("cli/tests/repl/help.in")),
+        Behavior::File(src_path("cli/tests/repl/help.out")),
         Behavior::Null,
     );
 }
@@ -462,8 +342,8 @@ fn test_repl_interactive() {
         bin_path("endbasic"),
         &[],
         0,
-        Behavior::File(src_path("tests/repl/interactive.in")),
-        Behavior::File(src_path("tests/repl/interactive.out")),
+        Behavior::File(src_path("cli/tests/repl/interactive.in")),
+        Behavior::File(src_path("cli/tests/repl/interactive.out")),
         Behavior::Null,
     );
 }
@@ -471,18 +351,14 @@ fn test_repl_interactive() {
 #[test]
 fn test_repl_load_save() {
     let dir = tempfile::tempdir().unwrap();
-    fs::copy(
-        &src_path("tests/lang/hello.bas"),
-        &dir.path().join("hello.bas"),
-    )
-    .unwrap();
+    fs::copy(&src_path("cli/tests/repl/hello.bas"), &dir.path().join("hello.bas")).unwrap();
     assert!(!dir.path().join("hello2.bas").exists());
     check(
         bin_path("endbasic"),
         &["--programs-dir", dir.path().to_str().unwrap()],
         0,
-        Behavior::File(src_path("tests/repl/load-save.in")),
-        Behavior::File(src_path("tests/repl/load-save.out")),
+        Behavior::File(src_path("cli/tests/repl/load-save.in")),
+        Behavior::File(src_path("cli/tests/repl/load-save.out")),
         Behavior::Null,
     );
     assert!(dir.path().join("hello2.bas").exists());
@@ -494,8 +370,8 @@ fn test_repl_state_sharing() {
         bin_path("endbasic"),
         &[],
         0,
-        Behavior::File(src_path("tests/repl/state-sharing.in")),
-        Behavior::File(src_path("tests/repl/state-sharing.out")),
+        Behavior::File(src_path("cli/tests/repl/state-sharing.in")),
+        Behavior::File(src_path("cli/tests/repl/state-sharing.out")),
         Behavior::Null,
     );
 }
